################################################################################
################################################################################
###
###  This file is automatically generated. Do not change this file! Changes
###  will get overwritten! Change the source file for "setup.py" instead.
###  This is either 'packageinfo.json' or 'packageinfo.jsonc'
###
################################################################################
################################################################################


from setuptools import setup

def readme():
	with open("README.md", "r", encoding="UTF-8-sig") as f:
		return f.read()

setup(
	author = "Jürgen Knauth",
	author_email = "pubsrc@binary-overflow.de",
	classifiers = [
		"Development Status :: 5 - Production/Stable",
		"License :: OSI Approved :: Apache Software License",
		"Programming Language :: Python :: 3",
		"Topic :: Software Development :: Testing",
	],
	description = "As the python exception API is quite a bit obscure this python module wraps around python exceptions to provide a clean interface for analysis and logging purposes.",
	include_package_data = False,
	install_requires = [
	],
	keywords = [
		"exception handling",
		"debugging",
	],
	license = "Apache2",
	name = "jk_exceptionhelper",
	packages = [
		"jk_exceptionhelper",
	],
	version = "0.2021.12.14",
	zip_safe = False,
	long_description = readme(),
	long_description_content_type="text/markdown",
)
